/*************************************************************************
 *
 *
 *    (c) Copyright Olimex 2010
 *
 *    File name   : taskIO.c
 *    Description : task for polling input/output
 *
 *    History :
 *    1. Date        : 20 July 2010
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"

/* DEFINE LOCAL TYPES HERE */

/* DEFINE LOCAL CONSTANTS HERE */

/* DEFINE LOCAL VARIABLES HERE */
static xSemaphoreHandle xMutexUartComm;
	
/* DECLARE EXTERNAL VARIABLES HERE */

/* DECLARE LOCAL FUNCTIONS HERE */
void vTaskIO( void * pvParameters );


// global var
xTaskHandle pxTaskIO;

/* DEFINE FUNCTIONS HERE */

char vTaskIOInitialize(void)
{
	char status = 0;
	
	xMutexUartComm = xSemaphoreCreateMutex();
	if(xMutexUartComm == NULL)
		status |= -1;
			
	// initialize UART module
	UartCommInit();

	// start task
	if( xTaskCreate( vTaskIO, "IO", configMINIMAL_STACK_SIZE + 200, NULL, tskIDLE_PRIORITY + 2, &pxTaskIO ) != pdPASS )
		status |= -1;

	return status;
}

/******************************************************************************
* Description: UartCommPrintSafe(..) - print string to the uart, thread safe
* Input: 	str - pointer to the string
* Output: 	none
* Return:	none
*******************************************************************************/
void UartCommPrintSafe(char *str)
{
	int len = strlen(str);
	
	xSemaphoreTake(xMutexUartComm, portMAX_DELAY);
	
	while(len) {
		int written;
		
		written = UartCommWrite(str, len);
		str += written;
		len -= written;
		
		if(len)
			vTaskDelay(1);
	}	
		
	xSemaphoreGive(xMutexUartComm);
}	



/********************************************************************/
/* DEFINE LOCAL FUNCTIONS HERE */

void vTaskIO( void * pvParameters )
{
	portTickType xLastWakeTime;
	int i;
	

	while(1)
	{
		ProcessIO ();

		// update buttons if possible ( PMP port should have been initialized! )
		if( xSemaphoreTake(xMutexParallelPort, 3) == pdTRUE) {
			WORD buttonsReadVal;

		    while(PMMODEbits.BUSY);
						
			ButtonsUpdate(buttonsReadVal, TRUE);
			xSemaphoreGive(xMutexParallelPort);
		}
		
		// service UART communication
		for(i = 0; i < 10; i++)
			if( !UartCommDoWork() )
				break;
		
		// sleep some time
		xLastWakeTime = xTaskGetTickCount();
		vTaskDelayUntil( &xLastWakeTime, 2 );
	}
}






