/*************************************************************************
 *
 *
 *    (c) Copyright Olimex 2009
 *
 *    File name   : taskTest.h
 *    Description : This task is for testing purposes and to indicate that RTOS is working properly
 *
 *    History :
 *    1. Date        : 26 June 2009
 *       Author      : Aleksandar Mitev
 *       Description : Create
 *
 **************************************************************************/
#include "includes.h"

/* DEFINE LOCAL TYPES HERE */

/* DEFINE LOCAL CONSTANTS HERE */

/* DEFINE LOCAL VARIABLES HERE */
static PROD_TEST_TYPE ProdTestType;
static char *strToPrint;
static xSemaphoreHandle xSemaphorePrint;

/* DECLARE EXTERNAL VARIABLES HERE */
extern xTaskHandle pxTaskIO;

/* DECLARE LOCAL FUNCTIONS HERE */
void vTaskTest( void * pvParameters );

/* DEFINE FUNCTIONS HERE */

void vTaskTestInitialize(void)
{
	portBASE_TYPE status;
	
	vSemaphoreCreateBinary(xSemaphorePrint);	
	if(xSemaphorePrint == NULL) {
		while(1);
	}
	
	ProdTestType = PROD_TEST_NONE;
	strToPrint = NULL;

	// start task
	status = xTaskCreate( vTaskTest, "TEST", configMINIMAL_STACK_SIZE, NULL, tskIDLE_PRIORITY + 1, NULL );
	Nop();
	Nop();

}

void vTaskTestStartTest(PROD_TEST_TYPE testType)
{
	ProdTestType = testType;
	
	xSemaphoreGive(xSemaphorePrint);
}

void vTaskTestContinueTest(void)
{
	xSemaphoreGive(xSemaphorePrint);
}

BOOL vTaskTestCheckTestCompleted(void)
{
	return (ProdTestType == PROD_TEST_NONE);
}

char* vTaskTestGetPrintStr(void)
{
	return strToPrint;
}



void vTaskTestPrint(char *str)
{
	strToPrint = str;
	
	// now wait to be enabled to continue
	xSemaphoreTake(xSemaphorePrint, portMAX_DELAY);
	strToPrint = NULL;
}

/********************************************************************/
/* DEFINE LOCAL FUNCTIONS HERE */

void vTaskTest( void * pvParameters )
{
	// take semaphore to block on waiting it afterward
	xSemaphoreTake(xSemaphorePrint, portMAX_DELAY);

	while(1) {
		xSemaphoreTake(xSemaphorePrint, portMAX_DELAY);

		switch(ProdTestType) {
		case PROD_TEST_NONE:
			break;
		case PROD_TEST_EXTCON:
			vTaskSuspend(pxTaskIO);
			//ExtConTestAll();	// SPP
			UserPeriphInit(); // test messes with pin directions, init again
			vTaskResume(pxTaskIO);
			break;
		case PROD_TEST_USBOTG:
			break;
		case PROD_TEST_CAN:
			StatLEDSet(1, FALSE);
			StatLEDSet(2, FALSE);
			StatLEDSet(3, FALSE);
			{
				int counter = 4000;
				char ch;
				
//				CANtesting(1);
				while(counter--) {
					//CANtesting(0);
					if( UartCommRead(&ch, 1) ) {
						UartCommWrite(&ch, 1);
						ch = '*';
						UartCommWrite(&ch, 1);	
					}
					
					vTaskDelay(5);
				}
			}	
			StatLEDSet(1, FALSE);
			StatLEDSet(2, FALSE);
			StatLEDSet(3, TRUE);
			break;
		}
		
		ProdTestType = PROD_TEST_NONE;
	}
		
		
}






